@@ -3,6 +3,8 @@
         data  Bxu/0./, Byu/0./, Bzu/0./
         data  Exu/0./, Eyu/0./, Ezu/0./
         data  MagField/0/,  ElecField/0/
+        data  Excom1/1.d-3/, Excom2/1.d-3/
+	data  LPMeffect/.true./  
         data  Tcoef/5.d0/
         data  Tmin/1.d-3/
         data  EdepdEdx/.true./
