@@ -26,6 +26,7 @@ c    ***************
 	real*8 gamma, beta2, gbeta2
         integer i
         real*8 mgb2
+	include "ZdEdxSpec.h"
 c
 
 	if(aPtcl.fm.p(4) - aPtcl.mass .le. 1.d-6 ) then
@@ -33,6 +34,22 @@ c
         elseif( dx .le. 0.) then
 	   de = 0.
 	else
+	   if( Tupper  .gt.  urb.EmaxN ) then
+	      urb.Emax = urb.EmaxN
+	      urb.EmaxI = urb.EmaxIN 
+	      urb.IEmaxI = urb.IEmaxIN 
+	      urb.EmaxByEmaxI = urb.EmaxByEmaxIN
+	      urb.PreSigma3 =  urb.PreSigma3N
+	   else
+	      urb.Emax = Tupper
+	      urb.EmaxI = urb.Emax + urb.AveIoniz
+	      urb.IEmaxI = urb.AveIoniz * urb.EmaxI
+	      urb.EmaxByEmaxI = urb.Emax/urb.EmaxI
+	      urb.PreSigma3 = urb.EmaxByEmaxI /
+     *         (urb.AveIoniz * log(urb.EmaxI/urb.AveIoniz) )
+     *          *  urb.ExToIoniz
+	   endif
+
 	   gamma =aPtcl.fm.p(4)/aPtcl.mass
 	   beta2 = 1.0d0 - 1./gamma**2
 	   gbeta2 = gamma**2 * beta2
@@ -53,15 +70,14 @@ c
 		 goto 100
 	      endif
 	   enddo	
-c             GEANT considers the very thin case specially
-c            (like epUrbanThin below), but this seems no good.
-c             We don't use such version.
-c	   if(urb.AveN(1)+ urb.AveN(2) + urb.AveN(3) .lt. 4.6) then
+c            very thin cases must be treated specially
+c            by epUrbanThin below, 
+	   if(urb.AveN(1)+ urb.AveN(2) + urb.AveN(3) .lt. 4.6) then
 c               ! no energy loss probability is > 0.01. treat specially
-c	      call epUrbanThin(urb, avededx*dx)
-c	   else
+	      call epUrbanThin(urb, avededx*dx)
+	   else
 	      call epUrbanNorm(urb)
-c	   endif
+	   endif
  100	      continue
 	   de = urb.ElossTotal
 	endif
@@ -71,21 +87,37 @@ c       ------------------------------------------------
 	implicit none
 #include "Zurban.h"
 	record /urban/urb
-	real*8 avede  ! input. <dE/dX> *dX
-c          very thin pass case.
+	real*8 avede  ! input. <dE/dX> *dX  GeV
+c          very thin path case.
 c          interaction is assumed to be with outer electrons only.
 c          Their energy level is 10 eV.
         integer i
 	real*8 u
+c             original prescription
+c	urb.AveNThin = avede/urb.EOuter
 c
-	urb.AveNThin = avede/urb.EOuter
+	urb.AveNThin = avede* (1./urb.EOuter - 1./urb.Emax)
+     *    /log(urb.Emax/urb.EOuter)
 	call kpoisn(urb.AveNThin, urb.NThin)
 	urb.ElossTotal = 0.
 	do i = 1, urb.NThin
 	   call rndc(u)	
+c            original     
+c	   urb.ElossTotal = urb.ElossTotal + urb.EOuter/
+c     *         (1. - u*urb.Emax/(urb.Emax + urb.EOuter))
+
 	   urb.ElossTotal = urb.ElossTotal + urb.EOuter/
-     *         (1. - u*urb.Emax/(urb.Emax + urb.EOuter))
+     *         (1. - u*(1.-urb.EOuter/urb.Emax))
         enddo
+c           another method; but we need to know peak position
+c          we use approximate Lanau distriubution as given
+c           by ksplandau
+c
+c	real*8 b, c
+c	c = avede/...
+c	b = c*..
+c	call ksplandau(b, c, urb.ElossTotal)
+
         end
 c       --------------------------
         subroutine epUrbanNorm(urb)
@@ -171,7 +203,9 @@ c
 #include "Zmedia.h"
         record /epmedia/media  ! input.  media
         record /urban/urb !  outupt. Urban's consts.    = media.urb
-
+c
+c       EmaxN, EmaxIN, IEmaxIN, EmaxByEmaxIN, PreSigma3N; these are
+c         to replace Emax etc (those without N)
 c
         urb.LargeN3 = 16
 	urb.ExToIoniz = 0.4
@@ -181,16 +215,16 @@ c
         urb.Zeff = media.Zeff
 	urb.AveIoniz = media.I
 	urb.AveIoniz2 = urb.AveIoniz**2
-	urb.Emax = media.sh.tcut  ! in GeV unit
-	urb.EmaxI = urb.Emax + urb.AveIoniz
-	urb.IEmaxI = urb.AveIoniz * urb.EmaxI
-	urb.EmaxByEmaxI = urb.Emax/urb.EmaxI
+
+	urb.EmaxN = media.sh.tcut  ! in GeV unit
+	urb.EmaxIN = urb.EmaxN + urb.AveIoniz
+	urb.IEmaxIN = urb.AveIoniz * urb.EmaxIN
+	urb.EmaxByEmaxIN = urb.EmaxN/urb.EmaxIN
+	urb.PreSigma3N = urb.EmaxByEmaxIN /
+     *  (urb.AveIoniz * log(urb.EmaxIN/urb.AveIoniz) ) *
+     *   urb.ExToIoniz
 
 	urb.Smallc2 = urb.LargeN3
-
-	urb.PreSigma3 = urb.EmaxByEmaxI /
-     *  (urb.AveIoniz * log(urb.EmaxI/urb.AveIoniz) ) *
-     *   urb.ExToIoniz
 	if(urb.Zeff .le. 2.) then
 	    urb.OscStr(2) = 0.
 	else	
