@@ -30,10 +30,10 @@
          do while(.true.)
             if(menu .eq. 1) then
                if(subdnumber .eq. 0) then
-                  call epExecGene(Det.cmp, Det.nct)
+                  call epExecGene(Det.cmp)
                else
-                  call epExecGene(SubdArea(SubD(subdnumber).loc+1), 
-     *                            SubD(subdnumber).nct)
+c                  call epExecGene(SubD(subdnumber).cmp)
+                  call epExecGene(SubdArea(SubD(subdnumber).loc+1))
                endif
             elseif(menu .gt. 1 .and. menu .le. maxmenu) then
                if(subdnumber .eq. 0) then
@@ -694,7 +694,7 @@ c            show components which are contained by others
       enddo
       end
 c     ***********************
-      subroutine epExecGene(cmp, ncmp)
+      subroutine epExecGene(cmp)
 c         epexec for subd
       implicit none
 #include  "Zglobalc.h"
@@ -702,8 +702,8 @@ c         epexec for subd
 #include  "Zcnfig.h"
 #include "ZepDraw.h"
 
-      integer ncmp
-      record /Component/ cmp(ncmp)
+
+      record /Component/ cmp(*)
 
       integer klena, i, icon
       logical first, showusage 
@@ -714,16 +714,9 @@ c         epexec for subd
       data showusage/.true./
       save showusage
 
-c      call mediaCount(cmp, matdef, noOfMedia, compnToMn)
-      call mediaCount(cmp, ncmp, matdef, noOfMedia)
       call epOpenMediaFile   ! open media file
-c///////
-      do i = 1, ncmp
-         write(*,*) i, compnToMn(i), matdef(compnToMn(i)),
-     *    Det.Cn2media(i),  Det.cmp(i).matter, cmp(i).matter
-      enddo
-c////////
-      do i = 1, ncmp
+
+      do i = 1, NumComp
          nv = 0
          call epaccept(cmp, i, icon)
          if(icon .ne. 0) goto 500
@@ -1373,15 +1366,17 @@ c         select sub detector
 
       NumComp = SubD(subdnumber).nct
       NumWorld = SubD(subdnumber).nworld
-      nstruc = 0 
+      nstruc = 0
       do i = 1, NumComp
          do j = 1, nstruc
             if(strucarray(j) .eq. 
      *          SubdArea(SubD(subdnumber).loc+i).struc) then
+c     SubD(subdnumber).cmp(i).struc) then
                goto 10
             endif
          enddo
          nstruc = nstruc + 1
+c         strucarray(nstruc) =SubD(subdnumber).cmp(i).struc
          strucarray(nstruc) =
      *        SubdArea( SubD(subdnumber).loc+i ).struc
  10      continue
@@ -1491,7 +1486,7 @@ c    *********************
 
 c         read config file
       call eprcnf(configFile)
-
+      call mediaCount(matdef, noOfMedia, compnToMn)
 
       NumComp = Det.nct
       NumWorld = Det.nworld
@@ -1518,7 +1513,6 @@ c         read config file
          levelc(j) = levelc(j) + w
          maxlevel = max(maxlevel, Det.cmp(i).level)
       enddo
-      call mediaCount(Det.cmp, matdef, noOfMedia, compnToMn)
       end
       subroutine epMemoCnfgFile
 c          memorize current config file name in Work/.configrc
