@@ -130,9 +130,9 @@ c             init for 1 event
        Move.Abort = 0     !  if this become non zero for an event, 
                      !  the particle is discarded or event is discarded
 c                 user dependent init
-
        call ui1ev
 
+
 c               next call sets to icon=1,    if usetip is not
 c               user made.
        call usetip(icon)
@@ -768,21 +768,37 @@ c         take trace of paritcles
 #include  "ZepTrackv.h"
 #include  "Zcode.h"
 #include  "ZepManager.h"
+#if defined NEXT486
+#define IMAG_P dimag
+#elif defined PCLinux
+#define IMAG_P dimag
+#else
+#define IMAG_P imag
+#endif
+      
 
 c          standard Trace information
       record /epPos/ posbw, posw
       real*8 xxx/-1.d37/, yyy/-1.d36/, zzz/1.d34/ 
       integer kkk/-1000/, chkchg/-1000/
-
+      real*8 erg
       integer k
       save xxx, yyy, zzz, kkk, chkchg
+
 c
 c          Trace informaion  ccc if for putting only electorns
 c
 c             convert to world coord. starting point of the segment
-         call epl2w(Cn, cTrack.pos, posbw)
- 
-         k = cTrack.p.code
+
+      erg= Move.Track.p.fm.p(4)-Move.Track.p.mass
+      if( TraceErg(1) .le. erg .and.  erg .le. TraceErg(2) ) goto 100
+      if( TraceErg(3) .le. erg .and.  erg .le. TraceErg(4) ) goto 100
+      if( TraceErg(5) .le. erg .and.  erg .le. TraceErg(6) ) goto 100
+      return  !  ***************
+ 100  continue
+      call epl2w(Cn, cTrack.pos, posbw)
+      
+      k = cTrack.p.code
 ccc      if(k .eq. kelec) then
          if(kkk .ne. k .or. xxx .ne. posbw.x .or. 
      *                      yyy .ne. posbw.y .or.
@@ -796,13 +812,13 @@ ccc      if(k .eq. kelec) then
                write(abs(IoTrace),
      *          '(3g16.8, i4, g11.4, i4, g16.8)')
      *            posbw.x, posbw.y, posbw.z, k,
-     *            Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *            erg,
      *            cTrack.p.charge,
      *            cTrack.t
             else
                write(abs(IoTrace),'(3g16.8,i4,g11.4, i4)')
      *            posbw.x, posbw.y, posbw.z, k,
-     *            Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *            erg,               
      *            cTrack.p.charge
             endif
          endif
@@ -812,13 +828,13 @@ c            end point of  the segment
             write(abs(IoTrace),'(3g16.8,i4,g11.4, i4, 
      *         g16.8)')
      *           posw.x, posw.y, posw.z, k,
-     *           Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *           erg,
      *           Move.Track.p.charge,
      *           Move.Track.t
          else
             write(abs(IoTrace),'(3g16.8,i4,g11.4, i4)')
      *           posw.x, posw.y, posw.z, k,
-     *           Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *           erg, 
      *           Move.Track.p.charge
          endif
          xxx = posw.x
@@ -842,16 +858,17 @@ c
 c
       real*8 tcomp, tphot, tpair, tgp, tcoh,  t
       real*8 E, prob, xprob(5), txray(5)
+      real*8 xs
       real*8  pairmfp, dl, dx, tmpair, u
       real*4  xsec(5)  !  coh, incoh,  P.E  1/(g/cm2)
-      real*8  Excom1, Excom2
+c      real*8  Excom1, Excom2  !now in ZepTrackp.h
       integer icon
 c            where xcom data is used. 
 c       Excom1: compton/photo abs/coherent scat
 c       Excom2: pair; default is  not use xcom
 c              both must be < 100 GeV
-      data Excom1/1.d-3/, Excom2/1.d-3/
-      save Excom1, Excom2
+c      data Excom1/1.d-3/, Excom2/1.d-3/
+c      save Excom1, Excom2
 
       E = cTrack.p.fm.p(4)
       if(E .le. EupperBndCS) then
@@ -893,14 +910,12 @@ c               below 1MeV, use accurate xs.
          tpair=1.e35
       endif
       if(IncGp .eq. 1 .and. E .gt. 150.d-3) then
-c          set cosmos condition(TrackBefMove, TargetMassN)
-c                 target mass and Z; thi is not used but
-c              projectile is set
-c         call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
          call ep2cosCond
          call cfixModel( cTrack.p )
-         call cghPath(E,  tgp)  ! cosmos function
-         tgp = tgp * 0.1   !         to g/cm2
+         call cgpXsec(Media(MediaNo).A,  E, xs)    ! xs in mb
+         prob = xs*Media(MediaNo).mbtoPX0   ! prob/r.l
+         call rndc(u)
+         tgp = -log(u)/prob          ! sampled path in r.l
       else
          tgp=1.e35
       endif
@@ -1155,15 +1170,17 @@ c              positron
              ok=aTrack.p.fm.p(4) .gt. KEmin
              if(.not. ok) then
                 if(kbtest(Eabsorb, BitPositron)) then
-
                    Move.dE = aTrack.p.fm.p(4) + masele
 
                    SumDe = SumDe + Move.dE
-                   Move.dEeff = ke
+c /////            Move.dEeff = ke
+                   Move.dEeff = Move.dE
+c/////
                    Move.dEioni= Move.dE  
                    if(Det.cmp(Cn).CountDE .ge. 1) then
                       call userde(1, aTrack, Move, Media(MediaNo))
                    endif
+
                 endif
              else
                 if(aTrack.p.fm.p(4) .lt. masele*1.01d0 ) then
@@ -1601,9 +1618,9 @@ c      ************
 #include  "Zcnfig.h"
 #include  "Zmass.h"           
 
-       real*8 eout, cost, cs, sn, sint
+       real*8 eout, cost, cs, sn, sint, Exray
        logical kbtest
-       record /epTrack/ elec1
+       record /epTrack/ elec1, xray
 c
 c           get Photo-electron energy
 c       call epphotoEe(Media(MediaNo).pe,   < v8.0
@@ -1619,19 +1636,51 @@ c                  regard it as deposited in the media
           if(Det.cmp(Cn).CountDE .ge. 1) then
              call userde(1, cTrack, Move, Media(MediaNo))
           endif
+          Exray = 0.
+       else
+c             bit 1 is not on; characteristic x-ray emmission;
+c             This was neglected in v8.71 or earlier.
+c             we  assume 
+c                1)  p.e effect takes place for the largest possible
+c                    shell energy (Say, if Eg> K-shell energy, L-shell
+c                    p.e effect is neglected and all p.e effect  is assumed
+c                    to take for K-shell. 
+c                2)  For such p.e effect, vacancy of electron level is 
+c                    filled by X-ray emission ; No Auger electron emmission
+c                    is considered. 1)+2) are good approximation.
+         Exray = max( cTrack.p.fm.p(4) - (eout - masele), 0.d0)
+c//////////
+c         write(0,*) ' ----------Exray=',Exray, ' eout=',eout
+c////////////
        endif
-
+c         emitted electron
+       elec1 = cTrack
+       call cmkptc(kelec, regptcl, -1, elec1.p)
        call kcossn(cs,sn)
        sint = sqrt(1.-cost**2)
        elec1.w.x = cs*sint
        elec1.w.y = sn*sint
        elec1.w.z = cost
-       cTrack.p.fm.p(4) = eout 
-       call eptransVect(cTrack.w, elec1.w, cTrack.w)
-       call cmkptc(kelec, regptcl, -1, cTrack.p)
-       call epe2p(cTrack)
-       call eppush(cTrack)
-ccc       if(Move.Abort .ne. 0) call epempty
+       elec1.p.fm.p(4) = eout 
+       call eptransVect(cTrack.w, elec1.w, elec1.w)
+       call epe2p(elec1)
+       call eppush(elec1)
+c         emitted xray; assume isotropic
+       if( Exray  .gt.  0.) then
+          xray = cTrack
+          call cmkptc(kphoton, 0, 0, xray.p)
+          call rndc(cost)
+          cost = 2.0*cost-1.0
+          call kcossn(cs,sn)
+          sint = sqrt(1.-cost**2)
+          xray.w.x = cs*sint
+          xray.w.y = sn*sint
+          xray.w.z = cost
+          xray.p.fm.p(4) = Exray
+          call eptransVect(cTrack.w, xray.w, xray.w)
+          call epe2p(xray)
+          call eppush(xray)
+       endif
        end
 c      ************
        subroutine epbrem
